﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Xml.Serialization;
using System.IO;

namespace DocumentViewerWithThumbnails
{
    public partial class MainPage : UserControl
    {
        Document doc;
        WebClient imgDownloadWebClient;

        public MainPage()
        {
            InitializeComponent();

            WebClient webClient = new WebClient();
            webClient.DownloadStringCompleted += OnWebClientDownloadStringCompleted;
            webClient.DownloadStringAsync(new Uri("Document/Document.xml", UriKind.Relative));
        }

        void OnWebClientDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs args)
        {
            if (args.Error == null)
            {
                XmlSerializer serializer = new XmlSerializer(typeof(Document));
                StringReader reader = new StringReader(args.Result);
                doc = (Document)serializer.Deserialize(reader);
                reader.Close();
                reader.Dispose();

                itemsCtrl.ItemsSource = doc.Pages;
            }
        }

        void OnItemMouseLeftButtonDown(object sender, MouseButtonEventArgs args)
        {
            // Get StackPanel and Thumbnail being clicked
            StackPanel stack = sender as StackPanel;
            Image thumbnail = stack.Children[0] as Image;

            // Find where to orient expanding Image
            Point ptThumbnailOrigin = thumbnail.TransformToVisual(itemsCtrl).Transform(new Point());
            imgTrans.X = ptThumbnailOrigin.X;
            imgTrans.Y = ptThumbnailOrigin.Y;
            imgTransAnimaX.To = imgTrans.X;
            imgTransAnimaY.To = imgTrans.Y;

            // Get WrapPanel within ItemsControl
            ContentPresenter presenter = VisualTreeHelper.GetParent(stack) as ContentPresenter;
            WrapPanel wrap = VisualTreeHelper.GetParent(presenter) as WrapPanel;

            // Get Page corresponding to Thumbnail
            int index = wrap.Children.IndexOf(presenter);
            DocumentPage page = doc.Pages[index];

            // Temporarily set Image source and begin animation
            img.Source = thumbnail.Source;
            img.Visibility = Visibility.Visible;
            expand.Begin();

            // Commence download of actual image
            imgDownloadWebClient = new WebClient();
            imgDownloadWebClient.OpenReadCompleted += OnImageDownloadOpenReadCompleted;
            imgDownloadWebClient.OpenReadAsync(new Uri(page.FileName, UriKind.Relative));
        }

        void OnImageDownloadOpenReadCompleted(object sender, OpenReadCompletedEventArgs args)
        {
            BitmapImage bitmap = new BitmapImage();
            img.Source = bitmap;
            bitmap.SetSource(args.Result);
        }

        void OnImageMouseLeftButtonDown(object sender, MouseButtonEventArgs args)
        {
            if (imgDownloadWebClient.IsBusy)
                imgDownloadWebClient.CancelAsync();

            contract.Begin();
        }

        private void OnContractAnimationCompleted(object sender, EventArgs args)
        {
            img.Visibility = Visibility.Collapsed;
            img.Source = null;
        }
    }
}
